@@ -44,6 +44,10 @@
 #include <semaphore.h>
 #include <stdint.h>
 
+extern "C" {
+#include <sys/types.h>
+}
+
 #define LIBTYPE
 #define DECLSPEC
 
